#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const utils_1 = require("@graphql-tools/utils");
const url_loader_1 = require("@graphql-tools/url-loader");
const json_file_loader_1 = require("@graphql-tools/json-file-loader");
const graphql_file_loader_1 = require("@graphql-tools/graphql-file-loader");
const load_1 = require("@graphql-tools/load");
const fs = require("fs");
const rimraf = require("rimraf");
const yargs = require("yargs");
// yargs command options config
const { path, url, file } = yargs.options({
    path: { type: 'string', alias: 'p', demandOption: true },
    url: { type: 'string' },
    file: { type: 'string' }
}).check((argv, options) => {
    return true;
}).argv;
(() => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c;
    // load schema
    const schema = yield load_1.loadSchema(url || file, {
        loaders: [
            new url_loader_1.UrlLoader(),
            new json_file_loader_1.JsonFileLoader(),
            new graphql_file_loader_1.GraphQLFileLoader()
        ]
    });
    // clean dir
    rimraf.sync(path);
    fs.mkdirSync(path);
    // get operations fields
    const mutations = (_a = schema.getMutationType()) === null || _a === void 0 ? void 0 : _a.getFields();
    const queries = (_b = schema.getQueryType()) === null || _b === void 0 ? void 0 : _b.getFields();
    const subscriptions = (_c = schema.getSubscriptionType()) === null || _c === void 0 ? void 0 : _c.getFields();
    // generate files
    const promisses = [];
    mutations && promisses.push(generateFilesForFields(schema, mutations, 'mutation'));
    queries && promisses.push(generateFilesForFields(schema, queries, 'query'));
    subscriptions && promisses.push(generateFilesForFields(schema, subscriptions, 'subscription'));
    yield Promise.all(promisses);
}))();
function generateFilesForFields(schema, obj, kind) {
    return __awaiter(this, void 0, void 0, function* () {
        const promisses = [];
        const opsPath = `${path}/${kind === 'query' ? 'queries' : kind === 'mutation' ? 'mutations' : 'subscriptions'}`;
        // create operations directory if it doesnt exit
        if (!fs.existsSync(opsPath)) {
            fs.mkdirSync(opsPath);
        }
        // loop through each operation and create graphql file
        Object.keys(obj).forEach(field => {
            promisses.push(new Promise(resolve => {
                const ops = utils_1.buildOperationNodeForField({ schema, kind, field });
                const file = `${opsPath}/${field}.graphql`;
                const content = `${graphql_1.print(ops).replace(/^([\t\s]*)((?!\().)*(:\s)/gm, '$1').replace(/(\$\w)/g, (s) => s.toLowerCase())}`;
                fs.writeFile(file, content, () => resolve({ field: field.replace(/\.?([A-Z]+)/g, (x, y) => '_' + y).replace(/^_/, '').toUpperCase(), file }));
            }));
        });
        // create import file for operations
        const files = yield Promise.all(promisses);
        const lines = files.map(({ field, file }) => `export const ${field} = loader('${file.replace(path, '.')}')`).join(`\r\n`);
        fs.writeFileSync(`${opsPath}/index.ts`, `import { loader } from 'graphql.macro'\r\n${lines}`);
    });
}
//# sourceMappingURL=index.js.map